@@ -23,43 +23,31 @@
 /**
  * SECTION:gfileinfo
  * @short_description: File Information and Attributes
- * @include: gio/gio.h
+ * @include: gio.h
  * @see_also: #GFile, <link linkend="gio-GFileAttribute">GFileAttribute</link>
- *
+ * 
  * Functionality for manipulating basic metadata for files. #GFileInfo
- * implements methods for getting information that all files should
- * contain, and allows for manipulation of extended attributes.
+ * implements methods for getting information that all files should 
+ * contain, and allows for manipulation of extended attributes. 
  *
  * See <link linkend="gio-GFileAttribute">GFileAttribute</link> for more
  * information on how GIO handles file attributes.
  *
- * To obtain a #GFileInfo for a #GFile, use g_file_query_info() (or its
- * async variant). To obtain a #GFileInfo for a file input or output
- * stream, use g_file_input_stream_query_info() or
+ * To obtain a #GFileInfo for a #GFile, use g_file_query_info() (or its 
+ * async variant). To obtain a #GFileInfo for a file input or output 
+ * stream, use g_file_input_stream_query_info() or 
  * g_file_output_stream_query_info() (or their async variants).
  *
- * To change the actual attributes of a file, you should then set the
- * attribute in the #GFileInfo and call g_file_set_attributes_from_info()
- * or g_file_set_attributes_async() on a GFile.
- *
- * However, not all attributes can be changed in the file. For instance,
- * the actual size of a file cannot be changed via g_file_info_set_size().
- * You may call g_file_query_settable_attributes() and
- * g_file_query_writable_namespaces() to discover the settable attributes
- * of a particular file at runtime.
- *
- * #GFileAttributeMatcher allows for searching through a #GFileInfo for
+ * #GFileAttributeMatcher allows for searching through a #GFileInfo for 
  * attributes.
  **/
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 
 #include "gfileinfo.h"
-#include "gfileinfo-priv.h"
 #include "gfileattribute-priv.h"
-#include "gicon.h"
 #include "glibintl.h"
 
 #include "gioalias.h"
@@ -86,6 +74,9 @@ struct _GFileInfoClass
 };
 
 
+static gboolean g_file_attribute_matcher_matches_id (GFileAttributeMatcher *matcher,
+						     guint32 id);
+
 G_DEFINE_TYPE (GFileInfo, g_file_info, G_TYPE_OBJECT);
 
 typedef struct {
@@ -138,133 +129,6 @@ _lookup_namespace (const char *namespace)
   return ns_info;
 }
 
-static guint32
-_lookup_attribute (const char *attribute)
-{
-  guint32 attr_id, id;
-  char *ns;
-  const char *colon;
-  NSInfo *ns_info;
-  
-  attr_id = GPOINTER_TO_UINT (g_hash_table_lookup (attribute_hash, attribute));
-
-  if (attr_id != 0)
-    return attr_id;
-
-  colon = strstr (attribute, "::");
-  if (colon)
-    ns = g_strndup (attribute, colon - attribute);
-  else
-    ns = g_strdup ("");
-
-  ns_info = _lookup_namespace (ns);
-  g_free (ns);
-
-  id = ++ns_info->attribute_id_counter;
-  attributes[ns_info->id] = g_realloc (attributes[ns_info->id], (id + 1) * sizeof (char *));
-  attributes[ns_info->id][id] = g_strdup (attribute);
-  
-  attr_id = MAKE_ATTR_ID (ns_info->id, id);
-
-  g_hash_table_insert (attribute_hash, attributes[ns_info->id][id], GUINT_TO_POINTER (attr_id));
-
-  return attr_id;
-}
-
-static void
-ensure_attribute_hash (void)
-{
-  if (attribute_hash != NULL)
-    return;
-
-  ns_hash = g_hash_table_new (g_str_hash, g_str_equal);
-  attribute_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-#define REGISTER_ATTRIBUTE(name) G_STMT_START{\
-  guint _u = _lookup_attribute (G_FILE_ATTRIBUTE_ ## name); \
-  /* use for generating the ID: g_print ("#define G_FILE_ATTRIBUTE_ID_%s (%u + %u)\n", #name + 17, _u & ~ID_MASK, _u & ID_MASK); */ \
-  g_assert (_u == G_FILE_ATTRIBUTE_ID_ ## name); \
-}G_STMT_END
-
-  REGISTER_ATTRIBUTE (STANDARD_TYPE);
-  REGISTER_ATTRIBUTE (STANDARD_IS_HIDDEN);
-  REGISTER_ATTRIBUTE (STANDARD_IS_BACKUP);
-  REGISTER_ATTRIBUTE (STANDARD_IS_SYMLINK);
-  REGISTER_ATTRIBUTE (STANDARD_IS_VIRTUAL);
-  REGISTER_ATTRIBUTE (STANDARD_NAME);
-  REGISTER_ATTRIBUTE (STANDARD_DISPLAY_NAME);
-  REGISTER_ATTRIBUTE (STANDARD_EDIT_NAME);
-  REGISTER_ATTRIBUTE (STANDARD_COPY_NAME);
-  REGISTER_ATTRIBUTE (STANDARD_DESCRIPTION);
-  REGISTER_ATTRIBUTE (STANDARD_ICON);
-  REGISTER_ATTRIBUTE (STANDARD_CONTENT_TYPE);
-  REGISTER_ATTRIBUTE (STANDARD_FAST_CONTENT_TYPE);
-  REGISTER_ATTRIBUTE (STANDARD_SIZE);
-  REGISTER_ATTRIBUTE (STANDARD_ALLOCATED_SIZE);
-  REGISTER_ATTRIBUTE (STANDARD_SYMLINK_TARGET);
-  REGISTER_ATTRIBUTE (STANDARD_TARGET_URI);
-  REGISTER_ATTRIBUTE (STANDARD_SORT_ORDER);
-  REGISTER_ATTRIBUTE (ETAG_VALUE);
-  REGISTER_ATTRIBUTE (ID_FILE);
-  REGISTER_ATTRIBUTE (ID_FILESYSTEM);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_READ);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_WRITE);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_EXECUTE);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_DELETE);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_TRASH);
-  REGISTER_ATTRIBUTE (ACCESS_CAN_RENAME);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_MOUNT);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_UNMOUNT);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_EJECT);
-  REGISTER_ATTRIBUTE (MOUNTABLE_UNIX_DEVICE);
-  REGISTER_ATTRIBUTE (MOUNTABLE_UNIX_DEVICE_FILE);
-  REGISTER_ATTRIBUTE (MOUNTABLE_HAL_UDI);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_START);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_START_DEGRADED);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_STOP);
-  REGISTER_ATTRIBUTE (MOUNTABLE_START_STOP_TYPE);
-  REGISTER_ATTRIBUTE (MOUNTABLE_CAN_POLL);
-  REGISTER_ATTRIBUTE (MOUNTABLE_IS_MEDIA_CHECK_AUTOMATIC);
-  REGISTER_ATTRIBUTE (TIME_MODIFIED);
-  REGISTER_ATTRIBUTE (TIME_MODIFIED_USEC);
-  REGISTER_ATTRIBUTE (TIME_ACCESS);
-  REGISTER_ATTRIBUTE (TIME_ACCESS_USEC);
-  REGISTER_ATTRIBUTE (TIME_CHANGED);
-  REGISTER_ATTRIBUTE (TIME_CHANGED_USEC);
-  REGISTER_ATTRIBUTE (TIME_CREATED);
-  REGISTER_ATTRIBUTE (TIME_CREATED_USEC);
-  REGISTER_ATTRIBUTE (UNIX_DEVICE);
-  REGISTER_ATTRIBUTE (UNIX_INODE);
-  REGISTER_ATTRIBUTE (UNIX_MODE);
-  REGISTER_ATTRIBUTE (UNIX_NLINK);
-  REGISTER_ATTRIBUTE (UNIX_UID);
-  REGISTER_ATTRIBUTE (UNIX_GID);
-  REGISTER_ATTRIBUTE (UNIX_RDEV);
-  REGISTER_ATTRIBUTE (UNIX_BLOCK_SIZE);
-  REGISTER_ATTRIBUTE (UNIX_BLOCKS);
-  REGISTER_ATTRIBUTE (UNIX_IS_MOUNTPOINT);
-  REGISTER_ATTRIBUTE (DOS_IS_ARCHIVE);
-  REGISTER_ATTRIBUTE (DOS_IS_SYSTEM);
-  REGISTER_ATTRIBUTE (OWNER_USER);
-  REGISTER_ATTRIBUTE (OWNER_USER_REAL);
-  REGISTER_ATTRIBUTE (OWNER_GROUP);
-  REGISTER_ATTRIBUTE (THUMBNAIL_PATH);
-  REGISTER_ATTRIBUTE (THUMBNAILING_FAILED);
-  REGISTER_ATTRIBUTE (PREVIEW_ICON);
-  REGISTER_ATTRIBUTE (FILESYSTEM_SIZE);
-  REGISTER_ATTRIBUTE (FILESYSTEM_FREE);
-  REGISTER_ATTRIBUTE (FILESYSTEM_TYPE);
-  REGISTER_ATTRIBUTE (FILESYSTEM_READONLY);
-  REGISTER_ATTRIBUTE (FILESYSTEM_USE_PREVIEW);
-  REGISTER_ATTRIBUTE (GVFS_BACKEND);
-  REGISTER_ATTRIBUTE (SELINUX_CONTEXT);
-  REGISTER_ATTRIBUTE (TRASH_ITEM_COUNT);
-  REGISTER_ATTRIBUTE (TRASH_ORIG_PATH);
-  REGISTER_ATTRIBUTE (TRASH_DELETION_DATE);
-
-#undef REGISTER_ATTRIBUTE
-}
-
 static guint32
 lookup_namespace (const char *namespace)
 {
@@ -273,7 +137,11 @@ lookup_namespace (const char *namespace)
   
   G_LOCK (attribute_hash);
   
-  ensure_attribute_hash ();
+  if (attribute_hash == NULL)
+    {
+      ns_hash = g_hash_table_new (g_str_hash, g_str_equal);
+      attribute_hash = g_hash_table_new (g_str_hash, g_str_equal);
+    }
 
   ns_info = _lookup_namespace (namespace);
   id = 0;
@@ -298,12 +166,42 @@ get_attribute_for_id (int attribute)
 static guint32
 lookup_attribute (const char *attribute)
 {
-  guint32 attr_id;
+  guint32 attr_id, id;
+  char *ns;
+  const char *colon;
+  NSInfo *ns_info;
   
   G_LOCK (attribute_hash);
-  ensure_attribute_hash ();
+  if (attribute_hash == NULL)
+    {
+      ns_hash = g_hash_table_new (g_str_hash, g_str_equal);
+      attribute_hash = g_hash_table_new (g_str_hash, g_str_equal);
+    }
 
-  attr_id = _lookup_attribute (attribute);
+  attr_id = GPOINTER_TO_UINT (g_hash_table_lookup (attribute_hash, attribute));
+
+  if (attr_id != 0)
+    {
+      G_UNLOCK (attribute_hash);
+      return attr_id;
+    }
+
+  colon = strstr (attribute, "::");
+  if (colon)
+    ns = g_strndup (attribute, colon - attribute);
+  else
+    ns = g_strdup ("");
+
+  ns_info = _lookup_namespace (ns);
+  g_free (ns);
+
+  id = ++ns_info->attribute_id_counter;
+  attributes[ns_info->id] = g_realloc (attributes[ns_info->id], (id + 1) * sizeof (char *));
+  attributes[ns_info->id][id] = g_strdup (attribute);
+  
+  attr_id = MAKE_ATTR_ID (ns_info->id, id);
+
+  g_hash_table_insert (attribute_hash, attributes[ns_info->id][id], GUINT_TO_POINTER (attr_id));
   
   G_UNLOCK (attribute_hash);
   
@@ -322,12 +220,13 @@ g_file_info_finalize (GObject *object)
   attrs = (GFileAttribute *)info->attributes->data;
   for (i = 0; i < info->attributes->len; i++)
     _g_file_attribute_value_clear (&attrs[i].value);
-  g_array_free (info->attributes, TRUE);
+  g_array_free (info->attributes, TRUE);  
 
   if (info->mask != NO_ATTRIBUTE_MASK)
     g_file_attribute_matcher_unref (info->mask);
-
-  G_OBJECT_CLASS (g_file_info_parent_class)->finalize (object);
+  
+  if (G_OBJECT_CLASS (g_file_info_parent_class)->finalize)
+    (*G_OBJECT_CLASS (g_file_info_parent_class)->finalize) (object);
 }
 
 static void
@@ -393,9 +292,6 @@ g_file_info_copy_into (GFileInfo *src_info,
       _g_file_attribute_value_set (&dest[i].value, &source[i].value);
     }
 
-  if (dest_info->mask != NO_ATTRIBUTE_MASK)
-    g_file_attribute_matcher_unref (dest_info->mask);
-
   if (src_info->mask == NO_ATTRIBUTE_MASK)
     dest_info->mask = NO_ATTRIBUTE_MASK;
   else
@@ -437,6 +333,7 @@ g_file_info_set_attribute_mask (GFileInfo             *info,
   int i;
   
   g_return_if_fail (G_IS_FILE_INFO (info));
+  g_return_if_fail (mask != NULL);
   
   if (mask != info->mask)
     {
@@ -448,7 +345,7 @@ g_file_info_set_attribute_mask (GFileInfo             *info,
       for (i = 0; i < info->attributes->len; i++)
 	{
 	  attr = &g_array_index (info->attributes, GFileAttribute, i);
-	  if (!_g_file_attribute_matcher_matches_id (mask,
+	  if (!g_file_attribute_matcher_matches_id (mask,
 						    attr->attribute))
 	    {
 	      _g_file_attribute_value_clear (&attr->value);
@@ -501,7 +398,7 @@ g_file_info_find_place (GFileInfo  *info,
 {
   int min, max, med;
   GFileAttribute *attrs;
-  /* Binary search for the place where attribute would be, if it's
+  /* Binary search for the place where attribute would be, if its
      in the array */
 
   min = 0;
@@ -575,42 +472,6 @@ g_file_info_has_attribute (GFileInfo  *info,
   return value != NULL;
 }
 
-/**
- * g_file_info_has_namespace:
- * @info: a #GFileInfo.
- * @name_space: a file attribute namespace.
- *
- * Checks if a file info structure has an attribute in the
- * specified @name_space.
- *
- * Returns: %TRUE if @Ginfo has an attribute in @name_space,
- *     %FALSE otherwise.
- *
- * Since: 2.22
- **/
-gboolean
-g_file_info_has_namespace (GFileInfo  *info,
-			   const char *name_space)
-{
-  GFileAttribute *attrs;
-  guint32 ns_id;
-  int i;
-
-  g_return_val_if_fail (G_IS_FILE_INFO (info), FALSE);
-  g_return_val_if_fail (name_space != NULL, FALSE);
-
-  ns_id = lookup_namespace (name_space);
-
-  attrs = (GFileAttribute *)info->attributes->data;
-  for (i = 0; i < info->attributes->len; i++)
-    {
-      if (GET_NS (attrs[i].attribute) == ns_id)
-	return TRUE;
-    }
-
-  return FALSE;
-}
-
 /**
  * g_file_info_list_attributes:
  * @info: a #GFileInfo.
@@ -657,7 +518,7 @@ g_file_info_list_attributes (GFileInfo  *info,
  * Gets the attribute type for an attribute key.
  * 
  * Returns: a #GFileAttributeType for the given @attribute, or 
- * %G_FILE_ATTRIBUTE_TYPE_INVALID if the key is not set.
+ * %G_FILE_ATTRIBUTE_TYPE_INVALID if the key is invalid.
  **/
 GFileAttributeType
 g_file_info_get_attribute_type (GFileInfo  *info,
@@ -770,42 +631,6 @@ g_file_info_get_attribute_status (GFileInfo  *info,
   return G_FILE_ATTRIBUTE_STATUS_UNSET;
 }
 
-/**
- * g_file_info_set_attribute_status:
- * @info: a #GFileInfo
- * @attribute: a file attribute key
- * @status: a #GFileAttributeStatus
- *
- * Sets the attribute status for an attribute key. This is only
- * needed by external code that implement g_file_set_attributes_from_info()
- * or similar functions.
- *
- * The attribute must exist in @info for this to work. Otherwise %FALSE
- * is returned and @info is unchanged.
- *
- * Returns: %TRUE if the status was changed, %FALSE if the key was not set.
- *
- * Since: 2.22
- */
-gboolean
-g_file_info_set_attribute_status (GFileInfo  *info,
-				  const char *attribute,
-				  GFileAttributeStatus status)
-{
-  GFileAttributeValue *val;
-
-  g_return_val_if_fail (G_IS_FILE_INFO (info), 0);
-  g_return_val_if_fail (attribute != NULL && *attribute != '\0', 0);
-
-  val = g_file_info_find_value_by_name (info, attribute);
-  if (val)
-    {
-      val->status = status;
-      return TRUE;
-    }
-
-  return FALSE;
-}
 
 GFileAttributeValue *
 _g_file_info_get_attribute_value (GFileInfo  *info,
@@ -914,32 +739,6 @@ g_file_info_get_attribute_byte_string (GFileInfo  *info,
   return _g_file_attribute_value_get_byte_string (value);
 }
 
-/**
- * g_file_info_get_attribute_stringv:
- * @info: a #GFileInfo.
- * @attribute: a file attribute key.
- *
- * Gets the value of a stringv attribute. If the attribute does
- * not contain a stringv, %NULL will be returned.
- *
- * Returns: the contents of the @attribute value as a stringv, or
- * %NULL otherwise. Do not free.
- *
- * Since: 2.22
- **/
-char **
-g_file_info_get_attribute_stringv (GFileInfo  *info,
-				   const char *attribute)
-{
-  GFileAttributeValue *value;
-
-  g_return_val_if_fail (G_IS_FILE_INFO (info), NULL);
-  g_return_val_if_fail (attribute != NULL && *attribute != '\0', NULL);
-
-  value = g_file_info_find_value_by_name (info, attribute);
-  return _g_file_attribute_value_get_stringv (value);
-}
-
 /**
  * g_file_info_get_attribute_boolean:
  * @info: a #GFileInfo.
@@ -1067,7 +866,7 @@ g_file_info_create_value (GFileInfo *info,
   int i;
 
   if (info->mask != NO_ATTRIBUTE_MASK &&
-      !_g_file_attribute_matcher_matches_id (info->mask, attr_id))
+      !g_file_attribute_matcher_matches_id (info->mask, attr_id))
     return NULL;
   
   i = g_file_info_find_place (info, attr_id);
@@ -1087,18 +886,15 @@ g_file_info_create_value (GFileInfo *info,
     }
 }
 
-void
-_g_file_info_set_attribute_by_id (GFileInfo                 *info,
-                                  guint32                    attribute,
-                                  GFileAttributeType         type,
-                                  gpointer                   value_p)
+static GFileAttributeValue *
+g_file_info_create_value_by_name (GFileInfo *info,
+				  const char *attribute)
 {
-  GFileAttributeValue *value;
+  guint32 attr_id;
 
-  value = g_file_info_create_value (info, attribute);
+  attr_id = lookup_attribute (attribute);
 
-  if (value)
-    _g_file_attribute_value_set_from_pointer (value, type, value_p, TRUE);
+  return g_file_info_create_value (info, attr_id);
 }
 
 /**
@@ -1116,22 +912,15 @@ g_file_info_set_attribute (GFileInfo                 *info,
 			   GFileAttributeType         type,
 			   gpointer                   value_p)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_by_id (info, lookup_attribute (attribute), type, value_p);
-}
+  value = g_file_info_create_value_by_name (info, attribute);
 
-void
-_g_file_info_set_attribute_object_by_id (GFileInfo *info,
-                                         guint32    attribute,
-				         GObject   *attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
   if (value)
-    _g_file_attribute_value_set_object (value, attr_value);
+    _g_file_attribute_value_set_from_pointer (value, type, value_p, TRUE);
 }
 
 /**
@@ -1148,62 +937,15 @@ g_file_info_set_attribute_object (GFileInfo  *info,
 				  const char *attribute,
 				  GObject    *attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
   g_return_if_fail (G_IS_OBJECT (attr_value));
 
-  _g_file_info_set_attribute_object_by_id (info,
-                                           lookup_attribute (attribute),
-                                           attr_value);
-}
-
-void
-_g_file_info_set_attribute_stringv_by_id (GFileInfo *info,
-                                          guint32    attribute,
-				          char     **attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_stringv (value, attr_value);
-}
-
-/**
- * g_file_info_set_attribute_stringv:
- * @info: a #GFileInfo.
- * @attribute: a file attribute key.
- * @attr_value: a %NULL terminated string array
- *
- * Sets the @attribute to contain the given @attr_value,
- * if possible.
- *
- * Sinze: 2.22
- **/
-void
-g_file_info_set_attribute_stringv (GFileInfo  *info,
-				   const char *attribute,
-				   char      **attr_value)
-{
-  g_return_if_fail (G_IS_FILE_INFO (info));
-  g_return_if_fail (attribute != NULL && *attribute != '\0');
-  g_return_if_fail (attr_value != NULL);
-
-  _g_file_info_set_attribute_stringv_by_id (info, 
-                                            lookup_attribute (attribute),
-                                            attr_value);
-}
-
-void
-_g_file_info_set_attribute_string_by_id (GFileInfo  *info,
-                                         guint32     attribute,
-				         const char *attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
-  if (value)
-    _g_file_attribute_value_set_string (value, attr_value);
+    _g_file_attribute_value_set_object (value, attr_value);
 }
 
 /**
@@ -1220,25 +962,15 @@ g_file_info_set_attribute_string (GFileInfo  *info,
 				  const char *attribute,
 				  const char *attr_value)
 {
+  GFileAttributeValue *value;
+  
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
   g_return_if_fail (attr_value != NULL);
 
-  _g_file_info_set_attribute_string_by_id (info,
-                                           lookup_attribute (attribute),
-                                           attr_value);
-}
-
-void
-_g_file_info_set_attribute_byte_string_by_id (GFileInfo  *info,
-                                              guint32     attribute,
-				              const char *attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_byte_string (value, attr_value);
+    _g_file_attribute_value_set_string (value, attr_value);
 }
 
 /**
@@ -1255,25 +987,15 @@ g_file_info_set_attribute_byte_string (GFileInfo  *info,
 				       const char *attribute,
 				       const char *attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
   g_return_if_fail (attr_value != NULL);
 
-  _g_file_info_set_attribute_byte_string_by_id (info,
-                                                lookup_attribute (attribute),
-                                                attr_value);
-}
-
-void
-_g_file_info_set_attribute_boolean_by_id (GFileInfo *info,
-                                          guint32    attribute,
-				          gboolean   attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_boolean (value, attr_value);
+    _g_file_attribute_value_set_byte_string (value, attr_value);
 }
 
 /**
@@ -1290,24 +1012,14 @@ g_file_info_set_attribute_boolean (GFileInfo  *info,
 				   const char *attribute,
 				   gboolean    attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_boolean_by_id (info,
-                                            lookup_attribute (attribute),
-                                            attr_value);
-}
-
-void
-_g_file_info_set_attribute_uint32_by_id (GFileInfo *info,
-                                         guint32    attribute,
-				         guint32    attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_uint32 (value, attr_value);
+    _g_file_attribute_value_set_boolean (value, attr_value);
 }
 
 /**
@@ -1324,26 +1036,17 @@ g_file_info_set_attribute_uint32 (GFileInfo  *info,
 				  const char *attribute,
 				  guint32     attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_uint32_by_id (info,
-                                           lookup_attribute (attribute),
-                                           attr_value);
-}
-
-void
-_g_file_info_set_attribute_int32_by_id (GFileInfo *info,
-                                        guint32    attribute,
-				        gint32     attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_int32 (value, attr_value);
+    _g_file_attribute_value_set_uint32 (value, attr_value);
 }
 
+
 /**
  * g_file_info_set_attribute_int32:
  * @info: a #GFileInfo.
@@ -1358,24 +1061,14 @@ g_file_info_set_attribute_int32 (GFileInfo  *info,
                                  const char *attribute,
                                  gint32      attr_value)
 {
+  GFileAttributeValue *value;
+ 
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_int32_by_id (info,
-                                          lookup_attribute (attribute),
-                                          attr_value);
-}
-
-void
-_g_file_info_set_attribute_uint64_by_id (GFileInfo *info,
-                                         guint32    attribute,
-				         guint64    attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_uint64 (value, attr_value);
+    _g_file_attribute_value_set_int32 (value, attr_value);
 }
 
 /**
@@ -1392,24 +1085,14 @@ g_file_info_set_attribute_uint64 (GFileInfo  *info,
 				  const char *attribute,
 				  guint64     attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_uint64_by_id (info,
-                                           lookup_attribute (attribute),
-                                           attr_value);
-}
-
-void
-_g_file_info_set_attribute_int64_by_id (GFileInfo *info,
-                                        guint32    attribute,
-				        gint64     attr_value)
-{
-  GFileAttributeValue *value;
-
-  value = g_file_info_create_value (info, attribute);
+  value = g_file_info_create_value_by_name (info, attribute);
   if (value)
-    _g_file_attribute_value_set_int64 (value, attr_value);
+    _g_file_attribute_value_set_uint64 (value, attr_value);
 }
 
 /**
@@ -1427,12 +1110,14 @@ g_file_info_set_attribute_int64  (GFileInfo  *info,
 				  const char *attribute,
 				  gint64      attr_value)
 {
+  GFileAttributeValue *value;
+
   g_return_if_fail (G_IS_FILE_INFO (info));
   g_return_if_fail (attribute != NULL && *attribute != '\0');
 
-  _g_file_info_set_attribute_int64_by_id (info,
-                                          lookup_attribute (attribute),
-                                          attr_value);
+  value = g_file_info_create_value_by_name (info, attribute);
+  if (value)
+    _g_file_attribute_value_set_int64 (value, attr_value);
 }
 
 /* Helper getters */
@@ -1620,7 +1305,7 @@ g_file_info_get_icon (GFileInfo *info)
   
   value = g_file_info_find_value (info, attr);
   obj = _g_file_attribute_value_get_object (value);
-  if (G_IS_ICON (obj))
+  if (obj != NULL && G_IS_ICON (obj))
     return G_ICON (obj);
   return NULL;
 }
@@ -1631,7 +1316,7 @@ g_file_info_get_icon (GFileInfo *info)
  * 
  * Gets the file's content type.
  * 
- * Returns: a string containing the file's content type.
+ * Returns: a string containing the file's content type.s
  **/
 const char *
 g_file_info_get_content_type (GFileInfo *info)
@@ -2336,13 +2021,11 @@ matcher_matches_id (GFileAttributeMatcher *matcher,
   return FALSE;
 }
 
-gboolean
-_g_file_attribute_matcher_matches_id (GFileAttributeMatcher *matcher,
-                                      guint32                id)
+static gboolean
+g_file_attribute_matcher_matches_id (GFileAttributeMatcher *matcher,
+                                     guint32                id)
 {
-  /* We return a NULL matcher for an empty match string, so handle this */
-  if (matcher == NULL)
-    return FALSE;
+  g_return_val_if_fail (matcher != NULL, FALSE);
   
   if (matcher->all)
     return TRUE;
