@@ -46,10 +46,6 @@
 #include <sys/mount.h>
 #endif
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
 #if defined(HAVE_STATFS) && defined(HAVE_STATVFS)
 /* Some systems have both statfs and statvfs, pick the
    most "native" for these */
@@ -84,15 +80,10 @@
 #include "glibintl.h"
 
 #ifdef G_OS_WIN32
-#define _WIN32_WINNT 0x0500
 #include <windows.h>
 #include <io.h>
 #include <direct.h>
 
-#ifndef FILE_READ_ONLY_VOLUME
-#define FILE_READ_ONLY_VOLUME           0x00080000
-#endif
-
 #ifndef S_ISDIR
 #define S_ISDIR(m) (((m) & _S_IFMT) == _S_IFDIR)
 #endif
@@ -103,7 +94,7 @@
 
 #include "gioalias.h"
 
-static void g_local_file_file_iface_init (GFileIface *iface);
+static void g_local_file_file_iface_init (GFileIface       *iface);
 
 static GFileAttributeInfoList *local_writable_attributes = NULL;
 static GFileAttributeInfoList *local_writable_namespaces = NULL;
@@ -120,7 +111,7 @@ G_DEFINE_TYPE_WITH_CODE (GLocalFile, g_local_file, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (G_TYPE_FILE,
 						g_local_file_file_iface_init))
 
-static char *find_mountpoint_for (const char *file, dev_t dev);
+static char * find_mountpoint_for (const char *file, dev_t dev);
 
 static void
 g_local_file_finalize (GObject *object)
@@ -152,7 +143,6 @@ g_local_file_class_init (GLocalFileClass *klass)
   g_file_attribute_info_list_add (list,
 				  G_FILE_ATTRIBUTE_UNIX_MODE,
 				  G_FILE_ATTRIBUTE_TYPE_UINT32,
-				  G_FILE_ATTRIBUTE_INFO_COPY_WITH_FILE |
 				  G_FILE_ATTRIBUTE_INFO_COPY_WHEN_MOVED);
   
 #ifdef HAVE_CHOWN
@@ -224,7 +214,6 @@ canonicalize_filename (const char *filename)
 {
   char *canon, *start, *p, *q;
   char *cwd;
-  int i;
   
   if (!g_path_is_absolute (filename))
     {
@@ -237,24 +226,6 @@ canonicalize_filename (const char *filename)
 
   start = (char *)g_path_skip_root (canon);
 
-  /* POSIX allows double slashes at the start to
-   * mean something special (as does windows too).
-   * So, "//" != "/", but more than two slashes
-   * is treated as "/".
-   */
-  i = 0;
-  for (p = start - 1;
-       (p >= canon) &&
-	 G_IS_DIR_SEPARATOR (*p);
-       p--)
-    i++;
-  if (i > 2)
-    {
-      i -= 1;
-      start -= i;
-      memmove (start, start+i, strlen (start+i)+1);
-    }
-  
   p = start;
   while (*p != 0)
     {
@@ -377,14 +348,14 @@ name_is_valid_for_display (const char *string,
 			   gboolean    is_valid_utf8)
 {
   char c;
-
+  
   if (!is_valid_utf8 &&
       !g_utf8_validate (string, -1, NULL))
     return FALSE;
 
   while ((c = *string++) != 0)
     {
-      if (g_ascii_iscntrl (c))
+      if (g_ascii_iscntrl(c))
 	return FALSE;
     }
 
@@ -401,8 +372,7 @@ g_local_file_get_parse_name (GFile *file)
   char *roundtripped_filename;
   gboolean free_utf8_filename;
   gboolean is_valid_utf8;
-  char *escaped_path;
-  
+
   filename = G_LOCAL_FILE (file)->filename;
   if (get_filename_charset (&charset))
     {
@@ -432,6 +402,7 @@ g_local_file_get_parse_name (GFile *file)
 	}
     }
 
+
   if (utf8_filename != NULL &&
       name_is_valid_for_display (utf8_filename, is_valid_utf8))
     {
@@ -442,16 +413,7 @@ g_local_file_get_parse_name (GFile *file)
     }
   else
     {
-      escaped_path = g_uri_escape_string (filename,
-					  G_URI_RESERVED_CHARS_ALLOWED_IN_PATH_ELEMENT "/",
-					  TRUE);
-      parse_name = g_strconcat ("file://",
-				(*escaped_path != '/') ? "/" : "",
-				escaped_path,
-				NULL);
-      
-      g_free (escaped_path);
-
+      parse_name = g_filename_to_uri (filename, NULL, NULL);
       if (free_utf8_filename)
 	g_free (utf8_filename);
     }
@@ -513,19 +475,12 @@ match_prefix (const char *path,
   prefix_len = strlen (prefix);
   if (strncmp (path, prefix, prefix_len) != 0)
     return NULL;
-  
-  /* Handle the case where prefix is the root, so that
-   * the IS_DIR_SEPRARATOR check below works */
-  if (prefix_len > 0 &&
-      G_IS_DIR_SEPARATOR (prefix[prefix_len-1]))
-    prefix_len--;
-  
   return path + prefix_len;
 }
 
 static gboolean
-g_local_file_prefix_matches (GFile *parent,
-			     GFile *descendant)
+g_local_file_contains_file (GFile *parent,
+			    GFile *descendant)
 {
   GLocalFile *parent_local = G_LOCAL_FILE (parent);
   GLocalFile *descendant_local = G_LOCAL_FILE (descendant);
@@ -618,10 +573,6 @@ get_fs_type (long f_type)
     {
     case 0xadf5:
       return "adfs";
-    case 0x5346414f:
-      return "afs";
-    case 0x0187:
-      return "autofs";
     case 0xADFF:
       return "affs";
     case 0x42465331:
@@ -726,7 +677,7 @@ static gboolean
 device_equal (gconstpointer v1,
               gconstpointer v2)
 {
-  return *(dev_t *)v1 == *(dev_t *)v2;
+  return *(dev_t *)v1 == * (dev_t *)v2;
 }
 
 static guint
@@ -803,111 +754,6 @@ get_mount_info (GFileInfo             *fs_info,
 
 #endif
 
-#ifdef G_OS_WIN32
-
-static gboolean
-is_xp_or_later (void)
-{
-  static int result = -1;
-
-  if (result == -1)
-    {
-      OSVERSIONINFOEX ver_info = {0};
-      DWORDLONG cond_mask = 0;
-      int op = VER_GREATER_EQUAL;
-
-      ver_info.dwOSVersionInfoSize = sizeof ver_info;
-      ver_info.dwMajorVersion = 5;
-      ver_info.dwMinorVersion = 1;
-
-      VER_SET_CONDITION (cond_mask, VER_MAJORVERSION, op);
-      VER_SET_CONDITION (cond_mask, VER_MINORVERSION, op);
-
-      result = VerifyVersionInfo (&ver_info,
-				  VER_MAJORVERSION | VER_MINORVERSION, 
-				  cond_mask) != 0;
-    }
-
-  return result;
-}
-
-static wchar_t *
-get_volume_for_path (const char *path)
-{
-  long len;
-  wchar_t *wpath;
-  wchar_t *result;
-
-  wpath = g_utf8_to_utf16 (path, -1, NULL, &len, NULL);
-  result = g_new (wchar_t, len + 2);
-
-  if (!GetVolumePathNameW (wpath, result, len + 2))
-    {
-      char *msg = g_win32_error_message (GetLastError ());
-      g_critical ("GetVolumePathName failed: %s", msg);
-      g_free (msg);
-      g_free (result);
-      g_free (wpath);
-      return NULL;
-    }
-
-  len = wcslen (result);
-  if (len > 0 && result[len-1] != L'\\')
-    {
-      result = g_renew (wchar_t, result, len + 2);
-      result[len] = L'\\';
-      result[len + 1] = 0;
-    }
-
-  g_free (wpath);
-  return result;
-}
-
-static char *
-find_mountpoint_for (const char *file, dev_t dev)
-{
-  wchar_t *wpath;
-  char *utf8_path;
-
-  wpath = get_volume_for_path (file);
-  if (!wpath)
-    return NULL;
-
-  utf8_path = g_utf16_to_utf8 (wpath, -1, NULL, NULL, NULL);
-
-  g_free (wpath);
-  return utf8_path;
-}
-
-static void
-get_filesystem_readonly (GFileInfo  *info,
-			 const char *path)
-{
-  wchar_t *rootdir;
-
-  rootdir = get_volume_for_path (path);
-
-  if (rootdir)
-    {
-      if (is_xp_or_later ())
-        {
-          DWORD flags;
-          if (GetVolumeInformationW (rootdir, NULL, 0, NULL, NULL, &flags, NULL, 0))
-	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY,
-					       (flags & FILE_READ_ONLY_VOLUME) != 0);
-        }
-      else
-        {
-          if (GetDriveTypeW (rootdir) == DRIVE_CDROM)
-	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, TRUE);
-        }
-    }
-
-  g_free (rootdir);
-}
-
-#endif /* G_OS_WIN32 */
-
 static GFileInfo *
 g_local_file_query_filesystem_info (GFile         *file,
 				    const char    *attributes,
@@ -959,12 +805,10 @@ g_local_file_query_filesystem_info (GFile         *file,
 
   if (statfs_result == -1)
     {
-      int errsv = errno;
-
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Error getting filesystem info: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return NULL;
     }
 
@@ -1005,11 +849,7 @@ g_local_file_query_filesystem_info (GFile         *file,
 #endif
     }
 #ifdef USE_STATFS
-#if defined(HAVE_STRUCT_STATFS_F_FSTYPENAME)
-  fstype = g_strdup(statfs_buffer.f_fstypename);
-#else
   fstype = get_fs_type (statfs_buffer.f_type);
-#endif
   if (fstype &&
       g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_TYPE))
@@ -1020,7 +860,7 @@ g_local_file_query_filesystem_info (GFile         *file,
 					G_FILE_ATTRIBUTE_FILESYSTEM_READONLY))
     {
 #ifdef G_OS_WIN32
-      get_filesystem_readonly (info, local->filename);
+      /* need to implement with *unix_mount* */
 #else
       get_mount_info (info, local->filename, attribute_matcher);
 #endif
@@ -1043,22 +883,18 @@ g_local_file_find_enclosing_mount (GFile         *file,
 
   if (g_lstat (local->filename, &buf) != 0)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-		      /* Translators: This is an error message when trying to
-		       * find the enclosing (user visible) mount of a file, but
-		       * none exists. */
-		      _("Containing mount does not exist"));
+      g_set_error (error, G_IO_ERROR,
+		   G_IO_ERROR_NOT_FOUND,
+		   _("Containing mount does not exist"));
       return NULL;
     }
 
   mountpoint = find_mountpoint_for (local->filename, buf.st_dev);
   if (mountpoint == NULL)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-		      /* Translators: This is an error message when trying to
-		       * find the enclosing (user visible) mount of a file, but
-		       * none exists. */
-		      _("Containing mount does not exist"));
+      g_set_error (error, G_IO_ERROR,
+		   G_IO_ERROR_NOT_FOUND,
+		   _("Containing mount does not exist"));
       return NULL;
     }
 
@@ -1067,11 +903,9 @@ g_local_file_find_enclosing_mount (GFile         *file,
   if (mount)
     return mount;
 
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-		  /* Translators: This is an error message when trying to find
-		   * the enclosing (user visible) mount of a file, but none
-		   * exists. */
-		  _("Containing mount does not exist"));
+  g_set_error (error, G_IO_ERROR,
+	       G_IO_ERROR_NOT_FOUND,
+	       _("Containing mount does not exist"));
   return NULL;
 }
 
@@ -1095,7 +929,7 @@ g_local_file_set_display_name (GFile         *file,
       return NULL;
     }
   
-  new_file = g_file_get_child_for_display_name (parent, display_name, error);
+  new_file = g_file_get_child_for_display_name  (parent, display_name, error);
   g_object_unref (parent);
   
   if (new_file == NULL)
@@ -1113,7 +947,7 @@ g_local_file_set_display_name (GFile         *file,
       return NULL;
     }
 
-  if (g_rename (local->filename, new_local->filename) == -1)
+  if (rename (local->filename, new_local->filename) == -1)
     {
       errsv = errno;
 
@@ -1184,13 +1018,13 @@ g_local_file_query_writable_namespaces (GFile         *file,
 }
 
 static gboolean
-g_local_file_set_attribute (GFile                *file,
-			    const char           *attribute,
-			    GFileAttributeType    type,
-			    gpointer              value_p,
-			    GFileQueryInfoFlags   flags,
-			    GCancellable         *cancellable,
-			    GError              **error)
+g_local_file_set_attribute (GFile                      *file,
+			    const char                 *attribute,
+			    GFileAttributeType          type,
+			    gpointer                    value_p,
+			    GFileQueryInfoFlags         flags,
+			    GCancellable               *cancellable,
+			    GError                    **error)
 {
   GLocalFile *local = G_LOCAL_FILE (file);
 
@@ -1212,7 +1046,7 @@ g_local_file_set_attributes_from_info (GFile                *file,
 {
   GLocalFile *local = G_LOCAL_FILE (file);
   int res, chained_res;
-  GFileIface *default_iface;
+  GFileIface* default_iface;
 
   res = _g_local_file_info_set_attributes (local->filename,
 					   info, flags, 
@@ -1238,15 +1072,13 @@ g_local_file_read (GFile         *file,
   int fd;
   struct stat buf;
   
-  fd = g_open (local->filename, O_RDONLY|O_BINARY, 0);
+  fd = g_open (local->filename, O_RDONLY, 0);
   if (fd == -1)
     {
-      int errsv = errno;
-
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Error opening file: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return NULL;
     }
 
@@ -1305,18 +1137,10 @@ g_local_file_delete (GFile         *file,
   
   if (g_remove (local->filename) == -1)
     {
-      int errsv = errno;
-
-      /* Posix allows EEXIST too, but the more sane error
-	 is G_IO_ERROR_NOT_FOUND, and its what nautilus
-	 expects */
-      if (errsv == EEXIST)
-	errsv = ENOTEMPTY;
-
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Error removing file: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return FALSE;
     }
   
@@ -1390,7 +1214,6 @@ get_parent (const char *path,
   if (strcmp (parent, ".") == 0 ||
       strcmp (parent, path_copy) == 0)
     {
-      g_free (parent);
       g_free (path_copy);
       return NULL;
     }
@@ -1447,8 +1270,6 @@ expand_all_symlinks (const char *path)
   return res;
 }
 
-#ifndef G_OS_WIN32
-
 static char *
 find_mountpoint_for (const char *file, 
                      dev_t       dev)
@@ -1476,6 +1297,8 @@ find_mountpoint_for (const char *file,
     }
 }
 
+#ifndef G_OS_WIN32
+
 static char *
 find_topdir_for (const char *file)
 {
@@ -1489,6 +1312,8 @@ find_topdir_for (const char *file)
   return find_mountpoint_for (dir, dir_dev);
 }
 
+#endif
+
 static char *
 get_unique_filename (const char *basename, 
                      int         id)
@@ -1500,7 +1325,7 @@ get_unique_filename (const char *basename,
 
   dot = strchr (basename, '.');
   if (dot)
-    return g_strdup_printf ("%.*s.%d%s", (int)(dot - basename), basename, id, dot);
+    return g_strdup_printf ("%.*s.%d%s", dot - basename, basename, id, dot);
   else
     return g_strdup_printf ("%s.%d", basename, id);
 }
@@ -1581,79 +1406,6 @@ escape_trash_name (char *name)
   return g_string_free (str, FALSE);
 }
 
-gboolean
-_g_local_file_has_trash_dir (const char *dirname, dev_t dir_dev)
-{
-  static gsize home_dev_set = 0;
-  static dev_t home_dev;
-  char *topdir, *globaldir, *trashdir, *tmpname;
-  uid_t uid;
-  char uid_str[32];
-  struct stat global_stat, trash_stat;
-  gboolean res;
-  int statres;
-      
-  if (g_once_init_enter (&home_dev_set))
-    {
-      struct stat home_stat;
-      
-      g_stat (g_get_home_dir (), &home_stat);
-      home_dev = home_stat.st_dev;
-      g_once_init_leave (&home_dev_set, 1);
-    }
-
-  /* Assume we can trash to the home */
-  if (dir_dev == home_dev)
-    return TRUE;
-
-  topdir = find_mountpoint_for (dirname, dir_dev);
-  if (topdir == NULL)
-    return FALSE;
-
-  globaldir = g_build_filename (topdir, ".Trash", NULL); 
-  statres = g_lstat (globaldir, &global_stat);
- if (g_lstat (globaldir, &global_stat) == 0 &&
-      S_ISDIR (global_stat.st_mode) &&
-      (global_stat.st_mode & S_ISVTX) != 0)
-    {
-      /* got a toplevel sysadmin created dir, assume we
-       * can trash to it (we should be able to create a dir)
-       * This fails for the FAT case where the ownership of
-       * that dir would be wrong though..
-       */
-      g_free (globaldir);
-      g_free (topdir);
-      return TRUE;
-    }
-  g_free (globaldir);
-
-  /* No global trash dir, or it failed the tests, fall back to $topdir/.Trash-$uid */
-  uid = geteuid ();
-  g_snprintf (uid_str, sizeof (uid_str), "%lu", (unsigned long) uid);
-  
-  tmpname = g_strdup_printf (".Trash-%s", uid_str);
-  trashdir = g_build_filename (topdir, tmpname, NULL);
-  g_free (tmpname);
-
-  if (g_lstat (trashdir, &trash_stat) == 0)
-    {
-      g_free (topdir);
-      g_free (trashdir);
-      return
-	S_ISDIR (trash_stat.st_mode) &&
-	trash_stat.st_uid == uid;
-    }
-  g_free (trashdir);
-
-  /* User specific trash didn't exist, can we create it? */
-  res = g_access (topdir, W_OK) == 0;
-  
-  g_free (topdir);
-  
-  return res;
-}
-
-
 static gboolean
 g_local_file_trash (GFile         *file,
 		    GCancellable  *cancellable,
@@ -1670,17 +1422,17 @@ g_local_file_trash (GFile         *file,
   gboolean is_homedir_trash;
   char delete_time[32];
   int fd;
+#ifndef G_OS_WIN32
   struct stat trash_stat, global_stat;
   char *dirname, *globaldir;
+#endif
   
   if (g_lstat (local->filename, &file_stat) != 0)
     {
-      int errsv = errno;
-
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Error trashing file: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return FALSE;
     }
     
@@ -1697,13 +1449,14 @@ g_local_file_trash (GFile         *file,
       if (g_mkdir_with_parents (trashdir, 0700) < 0)
 	{
           char *display_name;
-          int errsv = errno;
+          int err;
 
+          err = errno;
           display_name = g_filename_display_name (trashdir);
           g_set_error (error, G_IO_ERROR,
-                       g_io_error_from_errno (errsv),
+                       g_io_error_from_errno (err),
                        _("Unable to create trash dir %s: %s"),
-                       display_name, g_strerror (errsv));
+                       display_name, g_strerror (err));
           g_free (display_name);
           g_free (trashdir);
           return FALSE;
@@ -1712,6 +1465,9 @@ g_local_file_trash (GFile         *file,
     }
   else
     {
+#ifdef G_OS_WIN32
+      g_warning ("Recycle bin not implemented");
+#else
       uid_t uid;
       char uid_str[32];
 
@@ -1755,47 +1511,28 @@ g_local_file_trash (GFile         *file,
 
       if (trashdir == NULL)
 	{
-	  gboolean tried_create;
-	  
 	  /* No global trash dir, or it failed the tests, fall back to $topdir/.Trash-$uid */
 	  dirname = g_strdup_printf (".Trash-%s", uid_str);
 	  trashdir = g_build_filename (topdir, dirname, NULL);
 	  g_free (dirname);
-
-	  tried_create = FALSE;
-
-	retry:
+	  
 	  if (g_lstat (trashdir, &trash_stat) == 0)
 	    {
 	      if (!S_ISDIR (trash_stat.st_mode) ||
 		  trash_stat.st_uid != uid)
 		{
-		  /* Remove the failed directory */
-		  if (tried_create)
-		    g_remove (trashdir);
-		  
 		  /* Not a directory or not owned by user, ignore */
 		  g_free (trashdir);
 		  trashdir = NULL;
 		}
 	    }
-	  else
+	  else if (g_mkdir (trashdir, 0700) == -1)
 	    {
-	      if (!tried_create &&
-		  g_mkdir (trashdir, 0700) != -1)
-		{
-		  /* Ensure that the created dir has the right uid etc.
-		     This might fail on e.g. a FAT dir */
-		  tried_create = TRUE;
-		  goto retry;
-		}
-	      else
-		{
-		  g_free (trashdir);
-		  trashdir = NULL;
-		}
+	      g_free (trashdir);
+	      trashdir = NULL;
 	    }
 	}
+#endif
 
       if (trashdir == NULL)
 	{
@@ -1847,17 +1584,15 @@ g_local_file_trash (GFile         *file,
 
   if (fd == -1)
     {
-      int errsv = errno;
-
       g_free (filesdir);
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
-		   _("Unable to create trashing info file: %s"),
-		   g_strerror (errsv));
+		   g_io_error_from_errno (errno),
+		   _("Unable to create trashed file: %s"),
+		   g_strerror (errno));
       return FALSE;
     }
 
@@ -1872,17 +1607,15 @@ g_local_file_trash (GFile         *file,
 
   if (g_rename (local->filename, trashfile) == -1)
     {
-      int errsv = errno;
-
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       g_free (trashfile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Unable to trash file: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return FALSE;
     }
 
@@ -1900,6 +1633,13 @@ g_local_file_trash (GFile         *file,
   g_free (original_name);
   g_free (topdir);
   
+#ifdef G_OS_WIN32
+  {
+    GTimeVal now;
+    g_get_current_time (&now);
+    strncpy (delete_time, g_time_val_to_iso8601 (&now), sizeof (delete_time));
+  }
+#else
   {
     time_t t;
     struct tm now;
@@ -1908,6 +1648,7 @@ g_local_file_trash (GFile         *file,
     delete_time[0] = 0;
     strftime(delete_time, sizeof (delete_time), "%Y-%m-%dT%H:%M:%S", &now);
   }
+#endif
 
   data = g_strdup_printf ("[Trash Info]\nPath=%s\nDeletionDate=%s\n",
 			  original_name_escaped, delete_time);
@@ -1921,55 +1662,6 @@ g_local_file_trash (GFile         *file,
   
   return TRUE;
 }
-#else /* G_OS_WIN32 */
-gboolean
-_g_local_file_has_trash_dir (const char *dirname, dev_t dir_dev)
-{
-  return FALSE;			/* XXX ??? */
-}
-
-static gboolean
-g_local_file_trash (GFile         *file,
-		    GCancellable  *cancellable,
-		    GError       **error)
-{
-  GLocalFile *local = G_LOCAL_FILE (file);
-  SHFILEOPSTRUCTW op = {0};
-  gboolean success;
-  wchar_t *wfilename;
-  long len;
-
-  wfilename = g_utf8_to_utf16 (local->filename, -1, NULL, &len, NULL);
-  /* SHFILEOPSTRUCT.pFrom is double-zero-terminated */
-  wfilename = g_renew (wchar_t, wfilename, len + 2);
-  wfilename[len + 1] = 0;
-
-  op.wFunc = FO_DELETE;
-  op.pFrom = wfilename;
-  op.fFlags = FOF_ALLOWUNDO;
-
-  success = SHFileOperationW (&op) == 0;
-
-  if (success && op.fAnyOperationsAborted)
-    {
-      if (cancellable && !g_cancellable_is_cancelled (cancellable))
-	g_cancellable_cancel (cancellable);
-      g_set_error (error, G_IO_ERROR,
-		   G_IO_ERROR_CANCELLED,
-		   _("Unable to trash file: %s"),
-		   _("Operation was cancelled"));
-      success = FALSE;
-    }
-  else if (!success)
-    g_set_error (error, G_IO_ERROR,
-		 G_IO_ERROR_FAILED,
-		 _("Unable to trash file: %s"),
-		 _("internal error"));
-
-  g_free (wfilename);
-  return success;
-}
-#endif /* G_OS_WIN32 */
 
 static gboolean
 g_local_file_make_directory (GFile         *file,
@@ -1978,7 +1670,7 @@ g_local_file_make_directory (GFile         *file,
 {
   GLocalFile *local = G_LOCAL_FILE (file);
   
-  if (g_mkdir (local->filename, 0777) == -1)
+  if (g_mkdir (local->filename, 0755) == -1)
     {
       int errsv = errno;
 
@@ -2054,38 +1746,24 @@ g_local_file_move (GFile                  *source,
 		   gpointer                progress_callback_data,
 		   GError                **error)
 {
-  GLocalFile *local_source, *local_destination;
+  GLocalFile *local_source = G_LOCAL_FILE (source);
+  GLocalFile *local_destination = G_LOCAL_FILE (destination);
   struct stat statbuf;
   gboolean destination_exist, source_is_dir;
   char *backup_name;
   int res;
-  off_t source_size;
-  
-  if (!G_IS_LOCAL_FILE (source) ||
-      !G_IS_LOCAL_FILE (destination))
-    {
-      /* Fall back to default move */
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "Move not supported");
-      return FALSE;
-    }
-  
-  local_source = G_LOCAL_FILE (source);
-  local_destination = G_LOCAL_FILE (destination);
-  
+
   res = g_lstat (local_source->filename, &statbuf);
   if (res == -1)
     {
-      int errsv = errno;
-
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errsv),
+		   g_io_error_from_errno (errno),
 		   _("Error moving file: %s"),
-		   g_strerror (errsv));
+		   g_strerror (errno));
       return FALSE;
     }
-
-  source_is_dir = S_ISDIR (statbuf.st_mode);
-  source_size = statbuf.st_size;
+  else
+    source_is_dir = S_ISDIR (statbuf.st_mode);
   
   destination_exist = FALSE;
   res = g_lstat (local_destination->filename, &statbuf);
@@ -2098,16 +1776,10 @@ g_local_file_move (GFile                  *source,
 	  /* Always fail on dirs, even with overwrite */
 	  if (S_ISDIR (statbuf.st_mode))
 	    {
-	      if (source_is_dir)
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_WOULD_MERGE,
-			     _("Can't move directory over directory"));
-              else
-		g_set_error (error,
-			     G_IO_ERROR,
-			     G_IO_ERROR_IS_DIRECTORY,
-			     _("Can't copy over directory"));
+	      g_set_error (error,
+			   G_IO_ERROR,
+			   G_IO_ERROR_WOULD_MERGE,
+			   _("Can't move directory over directory"));
 	      return FALSE;
 	    }
 	}
@@ -2116,7 +1788,7 @@ g_local_file_move (GFile                  *source,
 	  g_set_error (error,
 		       G_IO_ERROR,
 		       G_IO_ERROR_EXISTS,
-		       _("Target file exists"));
+		       _("Target file already exists"));
 	  return FALSE;
 	}
     }
@@ -2124,7 +1796,7 @@ g_local_file_move (GFile                  *source,
   if (flags & G_FILE_COPY_BACKUP && destination_exist)
     {
       backup_name = g_strconcat (local_destination->filename, "~", NULL);
-      if (g_rename (local_destination->filename, backup_name) == -1)
+      if (rename (local_destination->filename, backup_name) == -1)
 	{
       	  g_set_error (error,
 		       G_IO_ERROR,
@@ -2141,20 +1813,18 @@ g_local_file_move (GFile                  *source,
     {
       /* Source is a dir, destination exists (and is not a dir, because that would have failed
 	 earlier), and we're overwriting. Manually remove the target so we can do the rename. */
-      res = g_unlink (local_destination->filename);
+      res = unlink (local_destination->filename);
       if (res == -1)
 	{
-          int errsv = errno;
-
 	  g_set_error (error, G_IO_ERROR,
-		       g_io_error_from_errno (errsv),
+		       g_io_error_from_errno (errno),
 		       _("Error removing target file: %s"),
-		       g_strerror (errsv));
+		       g_strerror (errno));
 	  return FALSE;
 	}
     }
   
-  if (g_rename (local_source->filename, local_destination->filename) == -1)
+  if (rename (local_source->filename, local_destination->filename) == -1)
     {
       int errsv = errno;
 
@@ -2176,33 +1846,27 @@ g_local_file_move (GFile                  *source,
 		     _("Error moving file: %s"),
 		     g_strerror (errsv));
       return FALSE;
-    }
 
-  /* Make sure we send full copied size */
-  if (progress_callback)
-    progress_callback (source_size, source_size, progress_callback_data);
-  
+    }
   return TRUE;
 }
 
 static GFileMonitor*
 g_local_file_monitor_dir (GFile             *file,
 			  GFileMonitorFlags  flags,
-			  GCancellable      *cancellable,
-			  GError           **error)
+			  GCancellable      *cancellable)
 {
   GLocalFile* local_file = G_LOCAL_FILE(file);
-  return _g_local_directory_monitor_new (local_file->filename, flags, error);
+  return _g_local_directory_monitor_new (local_file->filename, flags);
 }
 
 static GFileMonitor*
 g_local_file_monitor_file (GFile             *file,
 			   GFileMonitorFlags  flags,
-			   GCancellable      *cancellable,
-			   GError           **error)
+			   GCancellable      *cancellable)
 {
   GLocalFile* local_file = G_LOCAL_FILE(file);
-  return _g_local_file_monitor_new (local_file->filename, flags, error);
+  return _g_local_file_monitor_new (local_file->filename, flags);
 }
 
 static void
@@ -2219,7 +1883,7 @@ g_local_file_file_iface_init (GFileIface *iface)
   iface->get_uri = g_local_file_get_uri;
   iface->get_parse_name = g_local_file_get_parse_name;
   iface->get_parent = g_local_file_get_parent;
-  iface->prefix_matches = g_local_file_prefix_matches;
+  iface->contains_file = g_local_file_contains_file;
   iface->get_relative_path = g_local_file_get_relative_path;
   iface->resolve_relative_path = g_local_file_resolve_relative_path;
   iface->get_child_for_display_name = g_local_file_get_child_for_display_name;
