@@ -35,6 +35,7 @@ struct _GAsyncQueue
   GMutex *mutex;
   GCond *cond;
   GQueue *queue;
+  GDestroyNotify item_free_func;
   guint waiting_threads;
   gint32 ref_count;
 };
@@ -60,9 +61,29 @@ g_async_queue_new (void)
   retval->queue = g_queue_new ();
   retval->waiting_threads = 0;
   retval->ref_count = 1;
+  retval->item_free_func = NULL;
   return retval;
 }
 
+/**
+ * g_async_queue_new_full:
+ * 
+ * Creates a new asynchronous queue with an initial reference count of 1 and
+ * sets up a destroy notify function that is used to free any remaining
+ * queue items when the queue is destroyed after the final unref.
+ *
+ * Return value: the new #GAsyncQueue.
+ *
+ * Since: 2.16
+ **/
+GAsyncQueue*
+g_async_queue_new_full (GDestroyNotify item_free_func)
+{
+  GAsyncQueue *async_queue = g_async_queue_new ();
+  async_queue->item_free_func = item_free_func;
+  return async_queue;
+}
+
 /**
  * g_async_queue_ref:
  * @queue: a #GAsyncQueue.
@@ -147,6 +168,8 @@ g_async_queue_unref (GAsyncQueue *queue)
       g_mutex_free (queue->mutex);
       if (queue->cond)
 	g_cond_free (queue->cond);
+      if (queue->item_free_func)
+        g_queue_foreach (queue->queue, (GFunc) queue->item_free_func, NULL);
       g_queue_free (queue->queue);
       g_free (queue);
     }
@@ -308,15 +331,18 @@ g_async_queue_push_sorted_unlocked (GAsyncQueue      *queue,
 			 data, 
 			 (GCompareDataFunc)g_async_queue_invert_compare, 
 			 &sd);
+  if (queue->waiting_threads > 0)
+    g_cond_signal (queue->cond);
 }
 
 static gpointer
-g_async_queue_pop_intern_unlocked (GAsyncQueue* queue, gboolean try, 
-				   GTimeVal *end_time)
+g_async_queue_pop_intern_unlocked (GAsyncQueue *queue, 
+				   gboolean     try, 
+				   GTimeVal    *end_time)
 {
   gpointer retval;
 
-  if (!g_queue_peek_tail (queue->queue))
+  if (!g_queue_peek_tail_link (queue->queue))
     {
       if (try)
 	return NULL;
@@ -327,18 +353,18 @@ g_async_queue_pop_intern_unlocked (GAsyncQueue* queue, gboolean try,
       if (!end_time)
         {
           queue->waiting_threads++;
-	  while (!g_queue_peek_tail (queue->queue))
-            g_cond_wait(queue->cond, queue->mutex);
+	  while (!g_queue_peek_tail_link (queue->queue))
+            g_cond_wait (queue->cond, queue->mutex);
           queue->waiting_threads--;
         }
       else
         {
           queue->waiting_threads++;
-          while (!g_queue_peek_tail (queue->queue))
+          while (!g_queue_peek_tail_link (queue->queue))
             if (!g_cond_timed_wait (queue->cond, queue->mutex, end_time))
               break;
           queue->waiting_threads--;
-          if (!g_queue_peek_tail (queue->queue))
+          if (!g_queue_peek_tail_link (queue->queue))
 	    return NULL;
         }
     }
@@ -562,7 +588,7 @@ g_async_queue_length_unlocked (GAsyncQueue* queue)
  *
  * If you were sorting a list of priority numbers to make sure the
  * lowest priority would be at the top of the queue, you could use:
- * <informalexample><programlisting> 
+ * |[
  *  gint32 id1;
  *  gint32 id2;
  *   
@@ -570,7 +596,7 @@ g_async_queue_length_unlocked (GAsyncQueue* queue)
  *  id2 = GPOINTER_TO_INT (element2);
  *   
  *  return (id1 > id2 ? +1 : id1 == id2 ? 0 : -1);
- * </programlisting></informalexample>
+ * ]|
  *
  * Since: 2.10
  **/
@@ -622,5 +648,18 @@ g_async_queue_sort_unlocked (GAsyncQueue      *queue,
 		&sd);
 }
 
+/*
+ * Private API
+ */
+
+GMutex*
+_g_async_queue_get_mutex (GAsyncQueue* queue)
+{
+  g_return_val_if_fail (queue, NULL);
+  g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, NULL);
+
+  return queue->mutex;
+}
+
 #define __G_ASYNCQUEUE_C__
 #include "galiasdef.c"
